@@ -4,12 +4,21 @@ import configparser
 import os
 import sys
 import sysconfig
-from collections.abc import Mapping
 from pathlib import Path
+from typing import TYPE_CHECKING
 
 from .._logging import logger
 
-__all__ = ["get_python_include_dir", "get_python_library", "get_cmake_platform"]
+if TYPE_CHECKING:
+    from collections.abc import Mapping
+
+__all__ = [
+    "get_cmake_platform",
+    "get_numpy_include_dir",
+    "get_python_include_dir",
+    "get_python_library",
+    "get_soabi",
+]
 
 
 TARGET_TO_PLAT = {
@@ -65,6 +74,9 @@ def get_python_library(env: Mapping[str, str], *, abi3: bool = False) -> Path |
             libpath = libdir / ldlibrary
             if Path(os.path.expandvars(libpath)).is_file():
                 return libpath
+            logger.warning("libdir/ldlibrary: {} is not a real file!", libpath)
+        else:
+            logger.warning("libdir: {} is not a directory", libdir)
 
     framework_prefix = sysconfig.get_config_var("PYTHONFRAMEWORKPREFIX")
     if framework_prefix and Path(framework_prefix).is_dir() and ldlibrary:
@@ -92,12 +104,11 @@ def get_host_platform() -> str:
     setuptools get_host_platform (without 3.8 aix compat).
     """
 
-    if sys.version_info < (3, 8):
-        if os.name == "nt":
-            if "(arm)" in sys.version.lower():
-                return "win-arm32"
-            if "(arm64)" in sys.version.lower():
-                return "win-arm64"
+    if sys.version_info < (3, 8) and os.name == "nt":
+        if "(arm)" in sys.version.lower():
+            return "win-arm32"
+        if "(arm64)" in sys.version.lower():
+            return "win-arm64"
 
     return sysconfig.get_platform()
 
@@ -108,10 +119,16 @@ def get_platform(env: Mapping[str, str] | None = None) -> str:
     """
     if env is None:
         env = os.environ
-    if sys.platform.startswith("win"):
+    if sysconfig.get_platform().startswith("win"):
         if "VSCMD_ARG_TGT_ARCH" in env:
+            logger.debug(
+                "Selecting {} or {} due to VSCMD_ARG_TARGET_ARCH",
+                TARGET_TO_PLAT.get(env["VSCMD_ARG_TGT_ARCH"]),
+                get_host_platform(),
+            )
             return TARGET_TO_PLAT.get(env["VSCMD_ARG_TGT_ARCH"]) or get_host_platform()
         if "arm64" in env.get("SETUPTOOLS_EXT_SUFFIX", "").lower():
+            logger.debug("Windows ARM targeted via SETUPTOOLS_EXT_SUFFIX")
             return "win-arm64"
     return get_host_platform()
 
@@ -122,3 +139,33 @@ def get_cmake_platform(env: Mapping[str, str] | None) -> str:
     """
     plat = get_platform(env)
     return PLAT_TO_CMAKE.get(plat, plat)
+
+
+def get_soabi(env: Mapping[str, str], *, abi3: bool = False) -> str:
+    if abi3:
+        return "" if sysconfig.get_platform().startswith("win") else "abi3"
+
+    # Cross-compile support
+    setuptools_ext_suffix = env.get("SETUPTOOLS_EXT_SUFFIX", "")
+    if setuptools_ext_suffix:
+        return setuptools_ext_suffix.rsplit(".", 1)[0].lstrip(".")
+
+    if sys.version_info < (3, 8, 7):
+        # See https://github.com/python/cpython/issues/84006
+        import distutils.sysconfig  # pylint: disable=deprecated-module
+
+        ext_suffix = distutils.sysconfig.get_config_var("EXT_SUFFIX")
+    else:
+        ext_suffix = sysconfig.get_config_var("EXT_SUFFIX")
+
+    assert isinstance(ext_suffix, str)
+    return ext_suffix.rsplit(".", 1)[0].lstrip(".")
+
+
+def get_numpy_include_dir() -> Path | None:
+    try:
+        import numpy as np
+    except ImportError:
+        return None
+
+    return Path(np.get_include())
