@@ -1,7 +1,9 @@
 from __future__ import annotations
 
 import copy
+import dataclasses
 import difflib
+import os
 import platform
 import re
 import sys
@@ -15,13 +17,14 @@ from .. import __version__
 from .._compat import tomllib
 from .._logging import logger, rich_print
 from ..errors import CMakeConfigError
-from .skbuild_model import ScikitBuildSettings
+from .skbuild_model import CMakeSettings, NinjaSettings, ScikitBuildSettings
 from .sources import ConfSource, EnvSource, SourceChain, TOMLSource
 
 if TYPE_CHECKING:
-    import os
     from collections.abc import Generator, Mapping
 
+    from .._compat.typing import Literal
+
 __all__ = ["SettingsReader"]
 
 
@@ -29,6 +32,16 @@ def __dir__() -> list[str]:
     return __all__
 
 
+def strtobool(value: str) -> bool:
+    """
+    Converts a environment variable string into a boolean value.
+    """
+    value = value.lower()
+    if value.isdigit():
+        return bool(int(value))
+    return value in {"y", "yes", "on", "true", "t"}
+
+
 def version_match(value: str, match: str, name: str) -> str:
     """
     Returns a non-empty string if a version matches a specifier.
@@ -43,18 +56,28 @@ def regex_match(value: str, match: str) -> str:
     Returns a non-empty string if a value matches a regex.
     """
     did_match = re.compile(match).search(value) is not None
-    return f"{match!r} matched {value}" if did_match else ""
+    return f"{match!r} matched {value!r}" if did_match else ""
 
 
 def override_match(
+    *,
+    match_all: bool,
+    current_env: Mapping[str, str] | None,
+    current_state: Literal[
+        "sdist", "wheel", "editable", "metadata_wheel", "metadata_editable"
+    ],
     python_version: str | None = None,
     implementation_name: str | None = None,
     implementation_version: str | None = None,
     platform_system: str | None = None,
     platform_machine: str | None = None,
     platform_node: str | None = None,
+    env: dict[str, str] | None = None,
+    state: str | None = None,
 ) -> bool:
     matches = []
+    if current_env is None:
+        current_env = os.environ
 
     if python_version is not None:
         current_python_version = ".".join(str(x) for x in sys.version_info[:2])
@@ -92,31 +115,119 @@ def override_match(
         match_msg = regex_match(current_platform_node, platform_node)
         matches.append(match_msg)
 
+    if state is not None:
+        match_msg = regex_match(current_state, state)
+        matches.append(match_msg)
+
+    if env:
+        for key, value in env.items():
+            if isinstance(value, bool):
+                matches.append(
+                    f"env {key} is {value}"
+                    if strtobool(current_env.get(key, "")) == value
+                    else ""
+                )
+            elif key not in current_env:
+                matches.append("")
+            else:
+                current_value = current_env.get(key, "")
+                match_msg = regex_match(current_value, value)
+                matches.append(match_msg and f"env {key}: {match_msg}")
+
     if not matches:
         msg = "At least one override must be provided"
         raise ValueError(msg)
 
-    matched = all(matches)
-    if matched:
-        logger.info("Overrides {}", " and ".join(matches))
+    if match_all:
+        matched = all(matches)
+        if matched:
+            logger.info("Overrides {}", " and ".join(matches))
+    else:
+        matched = any(matches)
+        if matched:
+            logger.info("Overrides {}", " or ".join([m for m in matches if m]))
     return matched
 
 
+def _handle_minimum_version(
+    dc: CMakeSettings | NinjaSettings, minimum_version: Version | None
+) -> None:
+    """
+    Handle the minimum version option. Supports scikit-build-core < 0.8 style
+    minimum_version. Prints an error message and exits.
+    """
+    name = "cmake" if isinstance(dc, CMakeSettings) else "ninja"
+
+    version_default = next(
+        iter(f for f in dataclasses.fields(dc) if f.name == "version")
+    ).default
+
+    # Check for minimum_version < 0.8 and the modern version setting
+    if (
+        dc.version != version_default
+        and minimum_version is not None
+        and minimum_version < Version("0.8")
+    ):
+        rich_print(
+            f"[red][bold]ERROR:[/bold] Cannot set {name}.version if minimum-version is set to less than 0.8 (which is where it was introduced)"
+        )
+        raise SystemExit(7)
+
+    # Backwards compatibility for minimum_version
+    if dc.minimum_version is not None:
+        msg = f"Use {name}.version instead of {name}.minimum-version with scikit-build-core >= 0.8"
+        if minimum_version is None:
+            rich_print(f"[yellow][bold]WARNING:[/bold] {msg}")
+        elif minimum_version >= Version("0.8"):
+            rich_print(f"[red][bold]ERROR:[/bold] {msg}")
+            raise SystemExit(7)
+
+        if dc.version != version_default:
+            rich_print(
+                f"[red][bold]ERROR:[/bold] Cannot set both {name}.minimum_version and {name}.version; use version only for scikit-build-core >= 0.8."
+            )
+            raise SystemExit(7)
+
+        dc.version = SpecifierSet(f">={dc.minimum_version}")
+
+
 class SettingsReader:
     def __init__(
         self,
         pyproject: dict[str, Any],
         config_settings: Mapping[str, str | list[str]],
         *,
+        state: Literal[
+            "sdist", "wheel", "editable", "metadata_wheel", "metadata_editable"
+        ],
         verify_conf: bool = True,
+        env: Mapping[str, str] | None = None,
     ) -> None:
         pyproject = copy.deepcopy(pyproject)
 
         # Process overrides into the main dictionary if they match
         tool_skb = pyproject.get("tool", {}).get("scikit-build", {})
         for override in tool_skb.pop("overrides", []):
-            select = {k.replace("-", "_"): v for k, v in override.pop("if").items()}
-            if override_match(**select):
+            matched = True
+            if_override = override.pop("if", None)
+            if not if_override:
+                msg = "At least one 'if' override must be provided"
+                raise KeyError(msg)
+            if not isinstance(if_override, dict):
+                msg = "'if' override must be a table"
+                raise TypeError(msg)
+            if "any" in if_override:
+                any_override = if_override.pop("any")
+                select = {k.replace("-", "_"): v for k, v in any_override.items()}
+                matched = override_match(
+                    match_all=False, current_env=env, current_state=state, **select
+                )
+            select = {k.replace("-", "_"): v for k, v in if_override.items()}
+            if select:
+                matched = matched and override_match(
+                    match_all=True, current_env=env, current_state=state, **select
+                )
+            if matched:
                 for key, value in override.items():
                     if isinstance(value, dict):
                         for key2, value2 in value.items():
@@ -126,9 +237,16 @@ class SettingsReader:
                     else:
                         tool_skb[key] = value
 
+        prefixed = {
+            k: v for k, v in config_settings.items() if k.startswith("skbuild.")
+        }
+        remaining = {
+            k: v for k, v in config_settings.items() if not k.startswith("skbuild.")
+        }
         self.sources = SourceChain(
             EnvSource("SKBUILD"),
-            ConfSource(settings=config_settings, verify=verify_conf),
+            ConfSource("skbuild", settings=prefixed, verify=verify_conf),
+            ConfSource(settings=remaining, verify=verify_conf),
             TOMLSource("tool", "scikit-build", settings=pyproject),
             prefixes=["tool", "scikit-build"],
         )
@@ -144,9 +262,22 @@ class SettingsReader:
                 )
                 raise CMakeConfigError(msg)
 
+        if self.settings.editable.mode == "inplace":
+            if self.settings.editable.rebuild:
+                rich_print(
+                    "[red][bold]ERROR:[/bold] editable rebuild is incompatible with inplace mode"
+                )
+                raise SystemExit(7)
+
+            if self.settings.build_dir:
+                rich_print(
+                    "[red][bold]ERROR:[/bold] build-dir must be empty for editable inplace mode"
+                )
+                raise SystemExit(7)
+
         if self.settings.editable.rebuild and not self.settings.build_dir:
             rich_print(
-                "[red][bold]ERROR:[/bold] editable mode with rebuild requires build_dir"
+                "[red][bold]ERROR:[/bold] editable mode with rebuild requires build-dir"
             )
             raise SystemExit(7)
 
@@ -157,6 +288,9 @@ class SettingsReader:
         if self.settings.install.strip is None:
             self.settings.install.strip = install_policy
 
+        _handle_minimum_version(self.settings.cmake, self.settings.minimum_version)
+        _handle_minimum_version(self.settings.ninja, self.settings.minimum_version)
+
     def unrecognized_options(self) -> Generator[str, None, None]:
         return self.sources.unrecognized_options(ScikitBuildSettings)
 
@@ -174,8 +308,10 @@ class SettingsReader:
         return result
 
     def print_suggestions(self) -> None:
-        for index in (1, 2):
-            name = {1: "config-settings", 2: "pyproject.toml"}[index]
+        for index in (1, 2, 3):
+            name = {1: "config-settings", 2: "config-settings", 3: "pyproject.toml"}[
+                index
+            ]
             suggestions_dict = self.suggestions(index)
             if suggestions_dict:
                 rich_print(f"[red][bold]ERROR:[/bold] Unrecognized options in {name}:")
@@ -226,9 +362,14 @@ class SettingsReader:
         pyproject_path: os.PathLike[str] | str,
         config_settings: Mapping[str, str | list[str]] | None,
         *,
+        state: Literal[
+            "sdist", "wheel", "editable", "metadata_wheel", "metadata_editable"
+        ] = "sdist",
         verify_conf: bool = True,
     ) -> SettingsReader:
         with Path(pyproject_path).open("rb") as f:
             pyproject = tomllib.load(f)
 
-        return cls(pyproject, config_settings or {}, verify_conf=verify_conf)
+        return cls(
+            pyproject, config_settings or {}, verify_conf=verify_conf, state=state
+        )
