@@ -9,7 +9,7 @@ from types import SimpleNamespace
 
 import pytest
 
-from scikit_build_core.builder.builder import Builder
+from scikit_build_core.builder.builder import Builder, archs_to_tags, get_archs
 from scikit_build_core.builder.macos import get_macosx_deployment_target
 from scikit_build_core.builder.sysconfig import (
     get_python_include_dir,
@@ -23,7 +23,7 @@ from scikit_build_core.settings.skbuild_model import ScikitBuildSettings, WheelS
 # The envvar_higher case shouldn't happen, but the compiler should cause the
 # correct failure
 @pytest.mark.parametrize(
-    "pycom,envvar,answer",
+    ("pycom", "envvar", "answer"),
     [
         pytest.param("12.5.2", None, "12.0", id="only_plat_round"),
         pytest.param("10.12.2", None, "10.12", id="only_plat_classic"),
@@ -56,14 +56,14 @@ def test_get_python_library():
     pprint.pprint(sysconfig.get_config_vars())
 
     lib = get_python_library({})
-    if sys.platform.startswith("win"):
-        assert lib is None
-    else:
+    if sysconfig.get_platform().startswith("win"):
         assert lib
         assert lib.is_file()
+    else:
+        assert lib is None
 
 
-@pytest.mark.skipif(not sys.platform.startswith("win"), reason="Windows only")
+@pytest.mark.skipif(not sysconfig.get_platform().startswith("win"), reason="MSVC only")
 def test_get_python_library_xcompile(tmp_path):
     config_path = tmp_path / "tmp.cfg"
     config_path.write_text(
@@ -83,16 +83,11 @@ library_dirs=C:\\Python\\libs
     assert lib2 == Path("C:\\Python\\libs\\python3.lib")
 
 
-@pytest.mark.parametrize("archs", (["x86_64"], ["arm64", "universal2"]))
+@pytest.mark.parametrize("archs", [["x86_64"], ["arm64", "universal2"]])
 def test_builder_macos_arch(monkeypatch, archs):
     archflags = " ".join(f"-arch {arch}" for arch in archs)
     monkeypatch.setattr(sys, "platform", "darwin")
-    monkeypatch.setenv("ARCHFLAGS", archflags)
-    tmpcfg = SimpleNamespace(env=os.environ.copy())
-    tmpbuilder = typing.cast(
-        Builder, SimpleNamespace(config=tmpcfg, settings=ScikitBuildSettings())
-    )
-    assert Builder.get_archs(tmpbuilder) == archs
+    assert get_archs({"ARCHFLAGS": archflags}) == archs
 
 
 def test_builder_macos_arch_extra(monkeypatch):
@@ -105,11 +100,11 @@ def test_builder_macos_arch_extra(monkeypatch):
         settings=ScikitBuildSettings(wheel=WheelSettings()),
         config=tmpcfg,
     )
-    assert tmpbuilder.get_arch_tags() == ["universal2"]
+    assert archs_to_tags(get_archs(tmpbuilder.config.env)) == ["universal2"]
 
 
 @pytest.mark.parametrize(
-    "minver,archs,answer",
+    ("minver", "archs", "answer"),
     [
         pytest.param("10.12", ["x86_64"], "macosx_10_12_x86_64", id="10.12_x86_64"),
         pytest.param("10.12", ["arm64"], "macosx_11_0_arm64", id="10.12_arm64"),
@@ -128,6 +123,17 @@ def test_wheel_tag(monkeypatch, minver, archs, answer):
     assert plat == answer
 
 
+@pytest.mark.parametrize("archs", ["x86_64" "arm64" "universal2"])
+def test_wheel_build_tag(monkeypatch, archs):
+    monkeypatch.setattr(sys, "platform", "darwin")
+    monkeypatch.setenv("MACOSX_DEPLOYMENT_TARGET", "10.12")
+    monkeypatch.setattr(platform, "mac_ver", lambda: ("10.9.2", "", ""))
+
+    tags = WheelTag.compute_best(archs, build_tag="1")
+    answer = str(tags).split("-")[0]
+    assert answer == "1"
+
+
 def test_wheel_tag_expand(monkeypatch):
     monkeypatch.setattr(sys, "platform", "darwin")
     monkeypatch.setenv("MACOSX_DEPLOYMENT_TARGET", "10.10")
