@@ -1,61 +1,39 @@
 from __future__ import annotations
 
-import functools
 import os
 import shutil
-from collections.abc import Generator
 from pathlib import Path
+from typing import TYPE_CHECKING
 
 import pytest
+from packaging.specifiers import SpecifierSet
 from packaging.version import Version
 
 from scikit_build_core.cmake import CMake, CMaker
 from scikit_build_core.errors import CMakeNotFoundError
 
+if TYPE_CHECKING:
+    from collections.abc import Generator
+
 DIR = Path(__file__).parent.resolve()
 
 
-@functools.lru_cache(1)
-def cmake_path() -> str:
-    try:
-        import cmake
-    except ModuleNotFoundError:
-        cmake_str = shutil.which("cmake")
-        print(cmake_str)
-        if cmake_str is None:
-            pytest.skip("cmake must be installed for this test")
-        return os.fspath(Path(cmake_str).resolve())
-
-    return os.fspath(Path(cmake.CMAKE_BIN_DIR) / "cmake")
-
-
 def configure_args(config: CMaker, *, init: bool = False) -> Generator[str, None, None]:
-
-    yield cmake_path()
     yield f"-S{config.source_dir}"
     yield f"-B{config.build_dir}"
 
+    if config.single_config:
+        yield f"-DCMAKE_BUILD_TYPE:STRING={config.build_type}"
+
     if init:
         cmake_init = config.build_dir / "CMakeInit.txt"
         yield f"-C{cmake_init}"
 
-    if config.single_config:
-        yield f"-DCMAKE_BUILD_TYPE={config.build_type}"
 
-    if config.module_dirs:
-        yield "-DCMAKE_MODULE_PATH:PATH={}".format(
-            ";".join(str(p).replace("\\", "/") for p in config.module_dirs)
-        )
-
-    if config.prefix_dirs:
-        yield "-DCMAKE_PREFIX_PATH:PATH={}".format(
-            ";".join(str(p).replace("\\", "/") for p in config.prefix_dirs)
-        )
-
-
-@pytest.mark.configure
+@pytest.mark.configure()
 def test_init_cache(fp, tmp_path):
-    fp.register([cmake_path(), "--version"], stdout="3.14.0")
+    fp.register([fp.program("cmake"), "--version"], stdout="3.14.0")
+    fp.register([fp.program("cmake3"), "--version"], stdout="3.14.0")
 
     config = CMaker(
         CMake.default_search(),
@@ -68,8 +46,9 @@ def test_init_cache(fp, tmp_path):
     )
 
     cmd = list(configure_args(config, init=True))
-    print("Registering:", *cmd)
-    fp.register(cmd)
+    print("Registering: cmake", *cmd)
+    fp.register([fp.program("cmake"), *cmd])
+    fp.register([fp.program("cmake3"), *cmd])
     config.configure()
 
     cmake_init = config.build_dir / "CMakeInit.txt"
@@ -77,27 +56,28 @@ def test_init_cache(fp, tmp_path):
     assert (
         cmake_init.read_text()
         == f"""\
-set(SKBUILD ON CACHE BOOL "")
-set(SKBUILD_VERSION [===[1.0.0]===] CACHE STRING "")
-set(SKBUILD_PATH [===[{source_dir_str}]===] CACHE PATH "")
+set(SKBUILD ON CACHE BOOL "" FORCE)
+set(SKBUILD_VERSION [===[1.0.0]===] CACHE STRING "" FORCE)
+set(SKBUILD_PATH [===[{source_dir_str}]===] CACHE PATH "" FORCE)
 """
     )
 
 
-@pytest.mark.configure
+@pytest.mark.configure()
 def test_too_old(fp, monkeypatch):
-    monkeypatch.setattr(shutil, "which", lambda x: None)
-    fp.register([cmake_path(), "--version"], stdout="3.14.0")
-    print(cmake_path)
+    monkeypatch.setattr(shutil, "which", lambda _: None)
+    fp.register([fp.program("cmake"), "--version"], stdout="3.14.0")
+    fp.register([fp.program("cmake3"), "--version"], stdout="3.14.0")
 
     with pytest.raises(CMakeNotFoundError) as excinfo:
-        CMake.default_search(minimum_version=Version("3.15"))
-    assert "Could not find CMake with version >= 3.15" in excinfo.value.args[0]
+        CMake.default_search(version=SpecifierSet(">=3.15"))
+    assert "Could not find CMake with version >=3.15" in excinfo.value.args[0]
 
 
-@pytest.mark.configure
+@pytest.mark.configure()
 def test_cmake_args(tmp_path, fp):
-    fp.register([cmake_path(), "--version"], stdout="3.15.0")
+    fp.register([fp.program("cmake"), "--version"], stdout="3.15.0")
+    fp.register([fp.program("cmake3"), "--version"], stdout="3.15.0")
 
     config = CMaker(
         CMake.default_search(),
@@ -108,17 +88,19 @@ def test_cmake_args(tmp_path, fp):
 
     cmd = list(configure_args(config))
     cmd.append("-DSOMETHING=one")
-    print("Registering:", *cmd)
-    fp.register(cmd)
+    print("Registering: cmake", *cmd)
+    fp.register([fp.program("cmake"), *cmd])
+    fp.register([fp.program("cmake3"), *cmd])
 
     config.configure(cmake_args=["-DSOMETHING=one"])
 
     assert len(fp.calls) == 2
 
 
-@pytest.mark.configure
+@pytest.mark.configure()
 def test_cmake_paths(tmp_path, fp):
-    fp.register([cmake_path(), "--version"], stdout="3.15.0")
+    fp.register([fp.program("cmake"), "--version"], stdout="3.15.0")
+    fp.register([fp.program("cmake3"), "--version"], stdout="3.15.0")
 
     config = CMaker(
         CMake.default_search(),
@@ -130,9 +112,20 @@ def test_cmake_paths(tmp_path, fp):
     )
 
     cmd = list(configure_args(config))
-    print("Registering:", *cmd)
-    fp.register(cmd)
+    print("Registering: cmake", *cmd)
+    fp.register([fp.program("cmake"), *cmd])
+    fp.register([fp.program("cmake3"), *cmd])
 
     config.configure()
 
     assert len(fp.calls) == 2
+
+
+def test_get_cmake_via_envvar(monkeypatch: pytest.MonkeyPatch, fp):
+    monkeypatch.setattr("shutil.which", lambda x: x)
+    cmake_path = Path("some-prog")
+    fp.register([cmake_path, "--version"], stdout="cmake version 3.20.0\n")
+    monkeypatch.setenv("CMAKE_EXECUTABLE", str(cmake_path))
+    result = CMake.default_search(env=os.environ)
+    assert result.cmake_path == cmake_path
+    assert result.version == Version("3.20.0")
