@@ -1,3 +1,4 @@
+import gzip
 import hashlib
 import shutil
 import sys
@@ -6,12 +7,11 @@ import time
 import zipfile
 from pathlib import Path
 
+import build.util
 import pytest
 
 from scikit_build_core.build import _file_processor, build_sdist, build_wheel
 
-DIR = Path(__file__).parent.resolve()
-HELLO_PEP518 = DIR / "packages/simple_pyproject_ext"
 ENTRYPOINTS = """\
 [one.two]
 three = four
@@ -24,7 +24,7 @@ guithing = a.b:c
 """
 METADATA = """\
 Metadata-Version: 2.1
-Name: cmake-example
+Name: CMake.Example
 Version: 0.0.1
 Requires-Python: >=3.7
 Provides-Extra: test
@@ -32,67 +32,65 @@ Requires-Dist: pytest>=6.0; extra == "test"
 """
 
 mark_hashes_different = pytest.mark.xfail(
-    sys.platform.startswith("win32") or sys.platform.startswith("cygwin"),
+    sys.platform.startswith(("win", "cygwin")),
     reason="hashes differ on Windows",
     strict=False,
 )
 
 
-def test_pep517_sdist(tmp_path, monkeypatch):
+def compute_uncompressed_hash(inp: Path) -> str:
+    with gzip.open(inp, "rb") as f:
+        return hashlib.sha256(f.read()).hexdigest()
 
-    dist = tmp_path.resolve() / "dist"
-    monkeypatch.chdir(HELLO_PEP518)
-    if Path("dist").is_dir():
-        shutil.rmtree("dist")
-    out = build_sdist(str(dist))
+
+@pytest.mark.usefixtures("package_simple_pyproject_ext")
+def test_pep517_sdist():
+    dist = Path("dist")
+    out = build_sdist("dist")
 
     (sdist,) = dist.iterdir()
-    assert "cmake-example-0.0.1.tar.gz" == sdist.name
+    assert sdist.name == "cmake_example-0.0.1.tar.gz"
     assert sdist == dist / out
 
     with tarfile.open(sdist) as f:
         file_names = set(f.getnames())
         assert file_names == {
-            f"cmake-example-0.0.1/{x}"
+            f"cmake_example-0.0.1/{x}"
             for x in (
                 "CMakeLists.txt",
                 "pyproject.toml",
                 "src/main.cpp",
                 "PKG-INFO",
+                "LICENSE",
             )
         }
-        pkg_info = f.extractfile("cmake-example-0.0.1/PKG-INFO")
+        pkg_info = f.extractfile("cmake_example-0.0.1/PKG-INFO")
         assert pkg_info
         pkg_info_contents = pkg_info.read().decode()
         assert pkg_info_contents == METADATA
 
 
 @mark_hashes_different
-def test_pep517_sdist_hash(tmp_path, monkeypatch):
-    dist = tmp_path.resolve() / "dist"
-    monkeypatch.chdir(HELLO_PEP518)
-    if Path("dist").is_dir():
-        shutil.rmtree("dist")
-    out = build_sdist(str(dist))
+def test_pep517_sdist_hash(monkeypatch, package_simple_pyproject_ext):
+    # Unset SOURCE_DATE_EPOCH in order to guarantee the hash match
+    monkeypatch.delenv("SOURCE_DATE_EPOCH", raising=False)
+    dist = Path("dist")
+    out = build_sdist("dist")
     sdist = dist / out
-    hash = hashlib.sha256(sdist.read_bytes()).hexdigest()
-    if sys.version_info < (3, 9):
-        assert (
-            hash == "3b4af3fbe3d4505415bb1e55bb2e49902f4633d371ae7288007d90eb1488bc4d"
-        )
-    else:
-        assert (
-            hash == "d373b8458ee37b176cfd03f0f3199b30fdb034bca465b2826392a6c3af85ca4c"
-        )
+    hash = compute_uncompressed_hash(sdist)
+    assert hash == package_simple_pyproject_ext.sdist_hash
+    mode = sdist.stat().st_mode
+    assert mode == 33188
+    with gzip.open(sdist, "rb") as f:
+        f.read()
+        assert f.mtime == 1667997441
 
 
-def test_pep517_sdist_time_hash(tmp_path, monkeypatch):
-    dist = tmp_path.resolve() / "dist"
-    monkeypatch.chdir(HELLO_PEP518)
-    if Path("dist").is_dir():
-        shutil.rmtree("dist")
+@pytest.mark.usefixtures("package_simple_pyproject_ext")
+def test_pep517_sdist_time_hash():
+    dist = Path("dist")
 
-    out = build_sdist(str(dist))
+    out = build_sdist("dist")
     sdist = dist / out
     hash1 = hashlib.sha256(sdist.read_bytes()).hexdigest()
 
@@ -110,13 +108,11 @@ def test_pep517_sdist_time_hash(tmp_path, monkeypatch):
     assert hash1 == hash2
 
 
-def test_pep517_sdist_time_hash_nonreproducable(tmp_path, monkeypatch):
-    dist = tmp_path.resolve() / "dist"
-    monkeypatch.chdir(HELLO_PEP518)
-    if Path("dist").is_dir():
-        shutil.rmtree("dist")
+@pytest.mark.usefixtures("package_simple_pyproject_ext")
+def test_pep517_sdist_time_hash_nonreproducable():
+    dist = Path("dist")
 
-    out = build_sdist(str(dist), {"sdist.reproducible": "false"})
+    out = build_sdist("dist", {"sdist.reproducible": "false"})
     sdist = dist / out
     hash1 = hashlib.sha256(sdist.read_bytes()).hexdigest()
 
@@ -135,12 +131,13 @@ def test_pep517_sdist_time_hash_nonreproducable(tmp_path, monkeypatch):
 
 @mark_hashes_different
 @pytest.mark.parametrize("reverse_order", [False, True])
-def test_pep517_sdist_time_hash_set_epoch(tmp_path, monkeypatch, reverse_order):
-    dist = tmp_path.resolve() / "dist"
-    monkeypatch.chdir(HELLO_PEP518)
-    monkeypatch.setenv("SOURCE_DATE_EPOCH", "12345")
-    if Path("dist").is_dir():
-        shutil.rmtree("dist")
+def test_pep517_sdist_time_hash_set_epoch(
+    monkeypatch, reverse_order, package_simple_pyproject_ext
+):
+    dist = Path("dist")
+    monkeypatch.setenv(
+        "SOURCE_DATE_EPOCH", package_simple_pyproject_ext.source_date_epoch
+    )
 
     _each_unignored_file = _file_processor.each_unignored_file
 
@@ -153,26 +150,16 @@ def test_pep517_sdist_time_hash_set_epoch(tmp_path, monkeypatch, reverse_order):
 
     out = build_sdist(str(dist), {"sdist.reproducible": "true"})
     sdist = dist / out
-    hash = hashlib.sha256(sdist.read_bytes()).hexdigest()
-    if sys.version_info < (3, 9):
-        assert (
-            hash == "99d54d3090f932f00b77f01d12d422aacb674a6a9afd6af24f7afaaab7c3082b"
-        )
-    else:
-        assert (
-            hash == "359e38eb01415440cc8f7d9ee09612d625b8906ca09cd08c9d56f3eb972b0110"
-        )
+    hash = compute_uncompressed_hash(sdist)
+    assert hash == package_simple_pyproject_ext.sdist_dated_hash
 
 
-@pytest.mark.compile
-@pytest.mark.configure
-def test_pep517_wheel(tmp_path, monkeypatch, virtualenv):
-    dist = tmp_path / "dist"
-    dist.mkdir()
-    monkeypatch.chdir(HELLO_PEP518)
-    if Path("dist").is_dir():
-        shutil.rmtree("dist")
-    out = build_wheel(str(dist))
+@pytest.mark.compile()
+@pytest.mark.configure()
+@pytest.mark.usefixtures("package_simple_pyproject_ext")
+def test_pep517_wheel(virtualenv):
+    dist = Path("dist")
+    out = build_wheel("dist", {"cmake.targets": ["cmake_example"]})  # Could leave empty
     (wheel,) = dist.glob("cmake_example-0.0.1-*.whl")
     assert wheel == dist / out
 
@@ -184,6 +171,7 @@ def test_pep517_wheel(tmp_path, monkeypatch, virtualenv):
             entry_points = p.joinpath(
                 "cmake_example-0.0.1.dist-info/entry_points.txt"
             ).read_text()
+            assert p.joinpath("cmake_example-0.0.1.dist-info/licenses/LICENSE").exists()
 
         assert len(file_names) == 2
         assert "cmake_example-0.0.1.dist-info" in file_names
@@ -196,7 +184,7 @@ def test_pep517_wheel(tmp_path, monkeypatch, virtualenv):
         print(entry_points == ENTRYPOINTS)
         assert 'Requires-Dist: pytest>=6.0; extra == "test"' in metadata
         assert "Metadata-Version: 2.1" in metadata
-        assert "Name: cmake-example" in metadata
+        assert "Name: CMake.Example" in metadata
         assert "Version: 0.0.1" in metadata
         assert "Requires-Python: >=3.7" in metadata
         assert "Provides-Extra: test" in metadata
@@ -214,17 +202,69 @@ def test_pep517_wheel(tmp_path, monkeypatch, virtualenv):
     assert add.strip() == "3"
 
 
+@pytest.mark.compile()
+@pytest.mark.configure()
+@pytest.mark.usefixtures("package_simple_pyproject_source_dir")
+def test_pep517_wheel_source_dir(virtualenv):
+    dist = Path("dist")
+    out = build_wheel("dist", config_settings={"skbuild.wheel.build-tag": "1foo"})
+    (wheel,) = dist.glob("cmake_example-0.0.1-*.whl")
+    assert wheel == dist / out
+
+    if sys.version_info >= (3, 8):
+        with wheel.open("rb") as f:
+            p = zipfile.Path(f)
+            file_names = [p.name for p in p.iterdir()]
+            metadata = p.joinpath("cmake_example-0.0.1.dist-info/METADATA").read_text()
+            wheel_metadata = p.joinpath(
+                "cmake_example-0.0.1.dist-info/WHEEL"
+            ).read_text()
+            entry_points = p.joinpath(
+                "cmake_example-0.0.1.dist-info/entry_points.txt"
+            ).read_text()
+            assert p.joinpath("cmake_example-0.0.1.dist-info/licenses/LICENSE").exists()
+
+        assert len(file_names) == 2
+        assert "cmake_example-0.0.1.dist-info" in file_names
+        file_names.remove("cmake_example-0.0.1.dist-info")
+        (so_file,) = file_names
+
+        assert so_file.startswith("cmake_example")
+        print("SOFILE:", so_file)
+
+        print(entry_points == ENTRYPOINTS)
+        assert 'Requires-Dist: pytest>=6.0; extra == "test"' in metadata
+        assert "Metadata-Version: 2.1" in metadata
+        assert "Name: CMake.Example" in metadata
+        assert "Version: 0.0.1" in metadata
+        assert "Requires-Python: >=3.7" in metadata
+        assert "Provides-Extra: test" in metadata
+
+        assert "Build: 1foo" in wheel_metadata
+        assert "Wheel-Version: 1.0" in wheel_metadata
+        assert "Generator: scikit-build" in wheel_metadata
+        assert "Root-Is-Purelib: false" in wheel_metadata
+
+    virtualenv.install(wheel)
+
+    version = virtualenv.execute(
+        "import cmake_example; print(cmake_example.__version__)",
+    )
+    assert version.strip() == "0.0.1"
+
+    add = virtualenv.execute(
+        "import cmake_example; print(cmake_example.add(1, 2))",
+    )
+    assert add.strip() == "3"
+
+
 @pytest.mark.skip(reason="Doesn't work yet")
-@pytest.mark.compile
-@pytest.mark.configure
-def test_pep517_wheel_time_hash(tmp_path, monkeypatch):
-    dist = tmp_path / "dist"
-    dist.mkdir()
-    monkeypatch.chdir(HELLO_PEP518)
+@pytest.mark.compile()
+@pytest.mark.configure()
+def test_pep517_wheel_time_hash(monkeypatch):
     monkeypatch.setenv("SOURCE_DATE_EPOCH", "12345")
-    if Path("dist").is_dir():
-        shutil.rmtree("dist")
-    out = build_wheel(str(dist))
+    dist = Path("dist")
+    out = build_wheel("dist")
     wheel = dist / out
     hash1 = hashlib.sha256(wheel.read_bytes()).hexdigest()
 
@@ -240,3 +280,22 @@ def test_pep517_wheel_time_hash(tmp_path, monkeypatch):
     hash2 = hashlib.sha256(wheel.read_bytes()).hexdigest()
 
     assert hash1 == hash2
+
+
+@pytest.mark.usefixtures("package_simple_pyproject_ext")
+def test_prepare_metdata_for_build_wheel():
+    metadata = build.util.project_wheel_metadata(str(Path.cwd()), isolated=False)
+    answer = {
+        "Metadata-Version": "2.1",
+        "Name": "CMake.Example",
+        "Version": "0.0.1",
+        "Requires-Python": ">=3.7",
+        "Provides-Extra": "test",
+        "Requires-Dist": 'pytest>=6.0; extra == "test"',
+        "License-File": "LICENSE",
+    }
+
+    for k, b in answer.items():
+        assert metadata[k] == b
+
+    assert len(metadata) == len(answer)
